define([
    "dojo/_base/declare",
    "dojo/when",
    "dojox/mvc/getStateful",
    "dojox/mvc/at",
    "dojo/_base/array",

    "aps/_View",
    "aps/Status",
    "aps/ResourceStore",
    "aps/xhr",

    "./displayError"
], function (
    declare,
    when,
    getStateful,
    at,
    arr,

    View,
    Status,
    Store,
    xhr,

    displayError
) {
    var page, grid, selectionArray, self, contextId;
    return declare(View, {
        init: function() {
            self = this;
            /* Create the data store */
            var vpsStore = new Store({
                apsType: "http://aps-standard.org/samples/sample-basic/vps/1.0",
                target: "/aps/2/resources/"
            });

            /* Initialize a model to collect resource usage */
            self.usageModel = getStateful({
                "cpu": { "limit": 0, "usage": 0 },
                "ram": { "limit": 0, "usage": 0 },
                "disk": { "limit": 0, "usage": 0 }
            });

            /* Handler for the *New* button */
            var add = function() {
                /* Start the process of creating a VPS by going to the first screen */
                aps.apsc.gotoView("vps-wizard");
            };

            var refresh = function() {
                xhr.get("/aps/2/resources/" + contextId + "/resourceCounting").then(function() {
                    xhr.get("/aps/2/resources/" + contextId).then(function(mgmtContext) {
                        self.usageModel.set("cpu", mgmtContext.cpuusagetotal);
                        self.usageModel.set("ram", mgmtContext.memoryusagetotal);
                        self.usageModel.set("disk", mgmtContext.diskusagetotal);
                        grid.refresh();
                    });
                });
            };

            function changeState(state, btn) {
                var counter = selectionArray.length;

                /* Clear the current messages on the screen */
                page.get("messageList").removeAll();

                arr.forEach(selectionArray, function(vpsId){
                    console.log("I'm trying to change state of VPS with id = [" + vpsId + "]");

                    /* Save the VPS properties */
                    when(xhr("/aps/2/resources/" + vpsId + "/" + state, {method: "GET",  handleAs: "text"}),
                        /* If success, process the next VPS until the list is empty */
                        function(){
                            console.log("State of VPS with id = [" + vpsId + "] changed");
                            selectionArray.splice(selectionArray.indexOf(vpsId),1);
                            grid.refresh();
                            if (--counter === 0) { btn.cancel(); } /* Remove busy state for button */
                        },
                        /* If failure, call the error handler */
                        function(e){
                            displayError(e);
                            if (--counter === 0) { btn.cancel(); }
                        }
                    );
                });
            }

            /* Create the on-click handler for the *Stop* button */
            var stop = function() {
                changeState("stop", this);
            };

            /* Create the on-click handler for the *Start* button */
            var start = function() {
                changeState("start", this);
            };

            /* Create the on-click handler for the *Delete* button */
            var remove = function() {
                var btn = this;
                /* Get confirmation from the user for the delete operation */
                if (!confirm("Are you sure you want delete VPSes?")) {
                    btn.cancel();
                    return;
                }

                var counter = selectionArray.length;

                /* Clear the current messages on the screen */
                page.get("messageList").removeAll();

                selectionArray.forEach(function(vpsId){
                    console.log("I'm trying to delete VPS with id = [" + vpsId + "]");

                    /* Remove the VPS from the APS controller DB */
                    when(vpsStore.remove(vpsId),
                        /* If success, process the next VPS until the list is empty */
                        function(){
                            console.log("VPS with id = [" + vpsId + "] removed");
                            selectionArray.splice(selectionArray.indexOf(vpsId),1);
                            grid.refresh();
                            refresh();
                            if (--counter === 0) { btn.cancel(); }
                        },
                        /* If failure, call the error handler */
                        function(e){
                            displayError(e);
                            if (--counter === 0) { btn.cancel(); }
                        }
                    );
                });
            };

            /* Define and return widgets */
            return [
                ["aps/Grid", {
                    id:                this.genId("srv_grid"),
                    store:             vpsStore,
                    selectionMode:     "multiple",
                    apsResourceViewId: "server-edit",
                    noDataText: "No servers provisioned",
                    noEntriesFoundText: "No servers meet your search criteria",
                    columns: [{
                        field: "name",
                        name: "Name",
                        type: "resourceName",
                        filter: true
                    }, {
                        field: "hardware.memory",
                        name: "RAM",
                        title: "RAM",
                        type: "integer",
                        filter: true
                    }, {
                        field: "hardware.diskspace",
                        name: "Disk Space",
                        title: "Disk",
                        type: "integer",
                        filter: true
                    }, {
                        field: "hardware.CPU.number",
                        name: "CPU",
                        type: "integer",
                        filter: true
                    }, {
                        field: "platform.OS.name",
                        name: "OS",
                        filter: true
                    }, {
                        field: "state",
                        name: "State",
                        filter: true
                    }, {
                        field: "retry",
                        name: "Provisioning Status",
                        renderCell: function (row, data) {
                            return new Status({
                                useIcon: true,
                                status: data >= 5 ? "ready" : "inprogress",
                                statusInfo: {
                                    ready: {
                                        label: "Ready",
                                        type: "success"
                                    },
                                    inprogress: {
                                        label: "In progress",
                                        type: "warning"
                                    }
                                }
                            });
                        }
                    }]}, [
                    ["aps/Toolbar", [
                        ["aps/ToolbarButton", {
                            id: this.genId("srv_new"),
                            iconClass:"fa-plus",
                            type: "primary",
                            label: "Add New VPS",
                            onClick: add
                        }],
                        ["aps/ToolbarButton", {
                            id: this.genId("srv_start"),
                            iconClass:"fa-play",
                            type: "success",
                            label: "Start",
                            requireItems: true,
                            onClick: start
                        }],
                        ["aps/ToolbarButton", {
                            id: this.genId("srv_stop"),
                            iconClass:"fa-pause",
                            type: "warning",
                            label: "Stop",
                            requireItems: true,
                            onClick: stop
                        }],
                        ["aps/ToolbarButton", {
                            id: this.genId("srv_delete"),
                            iconClass:"fa-trash",
                            type: "danger",
                            label: "Delete",
                            requireItems: true,
                            onClick: remove
                        }],
                        ["aps/ToolbarButton", {
                            id: this.genId("srv_refresh"),
                            iconClass:"fa-refresh",
                            label: "Refresh",
                            autoBusy: false,
                            onClick: refresh
                        }]
                    ]]
                ]],
                ["aps/Tiles", {
                    title: "Resource consumption"
                },[
                    ["aps/Tile", {
                        title: "CPU power",
                        gridSize: "md-4"
                    }, [
                        ["aps/UsageInfo", {
                            showPie: false,
                            value: at(self.usageModel.cpu, "usage").direction(at.from),
                            maximum: at(self.usageModel.cpu, "limit").direction(at.from),
                            textFirstNumber: '${value}',
                            description: 'core-hours',
                            hint: 'Limit: ${maximum}'
                        }]]],
                    ["aps/Tile", {
                        title: "Memory usage",
                        gridSize: "md-4"
                    }, [
                        ["aps/UsageInfo", {
                            showPie: false,
                            value: at(self.usageModel.ram, "usage").direction(at.from),
                            maximum: at(self.usageModel.ram, "limit").direction(at.from),
                            textFirstNumber: '${value}',
                            description: 'MB-hours',
                            hint: 'Limit: ${maximum}'
                        }]]],
                    ["aps/Tile", {
                        title: "Disk usage",
                        gridSize: "md-4"
                    }, [
                        ["aps/UsageInfo", {
                            textSuffix: "GB",
                            value: at(self.usageModel.disk, "usage").direction(at.from),
                            maximum: at(self.usageModel.disk, "limit").direction(at.from),
                            textFirstNumber: '${value}',
                            textSecondNumber: '${maximum}'
                        }]]]
                ]]
            ];
        },	// End of Init

        onContext: function(context) {
            grid = this.byId("srv_grid");
            page = this.byId("apsPageContainer");
            selectionArray = grid.get("selectionArray");
            var mgmtContext = context.vars.context;
            contextId = mgmtContext.aps.id;
            self.usageModel.set("cpu", mgmtContext.cpuusagetotal);
            self.usageModel.set("ram", mgmtContext.memoryusagetotal);
            self.usageModel.set("disk", mgmtContext.diskusagetotal);
            grid.refresh();
            aps.apsc.hideLoading();
        },

        onHide: function() {
            this.byId("srv_new").cancel();
        }

    });
});
